// This file was generated by gir (https://github.com/gtk-rs/gir)
// from
// from gir-files (https://github.com/gtk-rs/gir-files.git)
// DO NOT EDIT

mod action_row;
pub use self::action_row::ActionRowBuilder;
pub use self::action_row::{ActionRow, NONE_ACTION_ROW};

mod application_window;
pub use self::application_window::ApplicationWindowBuilder;
pub use self::application_window::{ApplicationWindow, NONE_APPLICATION_WINDOW};

mod avatar;
pub use self::avatar::Avatar;
pub use self::avatar::AvatarBuilder;

mod carousel;
pub use self::carousel::Carousel;
pub use self::carousel::CarouselBuilder;

mod carousel_indicator_dots;
pub use self::carousel_indicator_dots::CarouselIndicatorDots;
pub use self::carousel_indicator_dots::CarouselIndicatorDotsBuilder;

mod carousel_indicator_lines;
pub use self::carousel_indicator_lines::CarouselIndicatorLines;
pub use self::carousel_indicator_lines::CarouselIndicatorLinesBuilder;

mod clamp;
pub use self::clamp::Clamp;
pub use self::clamp::ClampBuilder;

mod combo_row;
pub use self::combo_row::ComboRowBuilder;
pub use self::combo_row::{ComboRow, NONE_COMBO_ROW};

mod deck;
pub use self::deck::DeckBuilder;
pub use self::deck::{Deck, NONE_DECK};

mod enum_value_object;
pub use self::enum_value_object::EnumValueObject;

mod expander_row;
pub use self::expander_row::ExpanderRowBuilder;
pub use self::expander_row::{ExpanderRow, NONE_EXPANDER_ROW};

#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
mod flap;
#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
pub use self::flap::Flap;
#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
pub use self::flap::FlapBuilder;

mod header_bar;
pub use self::header_bar::HeaderBarBuilder;
pub use self::header_bar::{HeaderBar, NONE_HEADER_BAR};

mod header_group;
pub use self::header_group::HeaderGroup;
pub use self::header_group::HeaderGroupBuilder;

mod header_group_child;
pub use self::header_group_child::HeaderGroupChild;

mod keypad;
pub use self::keypad::KeypadBuilder;
pub use self::keypad::{Keypad, NONE_KEYPAD};

mod leaflet;
pub use self::leaflet::LeafletBuilder;
pub use self::leaflet::{Leaflet, NONE_LEAFLET};

mod preferences_group;
pub use self::preferences_group::PreferencesGroupBuilder;
pub use self::preferences_group::{PreferencesGroup, NONE_PREFERENCES_GROUP};

mod preferences_page;
pub use self::preferences_page::PreferencesPageBuilder;
pub use self::preferences_page::{PreferencesPage, NONE_PREFERENCES_PAGE};

mod preferences_row;
pub use self::preferences_row::PreferencesRowBuilder;
pub use self::preferences_row::{PreferencesRow, NONE_PREFERENCES_ROW};

mod preferences_window;
pub use self::preferences_window::PreferencesWindowBuilder;
pub use self::preferences_window::{PreferencesWindow, NONE_PREFERENCES_WINDOW};

mod search_bar;
pub use self::search_bar::SearchBarBuilder;
pub use self::search_bar::{SearchBar, NONE_SEARCH_BAR};

mod squeezer;
pub use self::squeezer::Squeezer;
pub use self::squeezer::SqueezerBuilder;

#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
mod status_page;
#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
pub use self::status_page::StatusPage;
#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
pub use self::status_page::StatusPageBuilder;

mod swipe_group;
pub use self::swipe_group::SwipeGroup;

mod swipe_tracker;
pub use self::swipe_tracker::SwipeTracker;
pub use self::swipe_tracker::SwipeTrackerBuilder;

mod swipeable;
pub use self::swipeable::{Swipeable, NONE_SWIPEABLE};

#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
mod tab_bar;
#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
pub use self::tab_bar::TabBar;
#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
pub use self::tab_bar::TabBarBuilder;

#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
mod tab_page;
#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
pub use self::tab_page::TabPage;
#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
pub use self::tab_page::TabPageBuilder;

#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
mod tab_view;
#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
pub use self::tab_view::TabView;
#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
pub use self::tab_view::TabViewBuilder;

mod title_bar;
pub use self::title_bar::TitleBar;
pub use self::title_bar::TitleBarBuilder;

mod value_object;
pub use self::value_object::ValueObject;

mod view_switcher;
pub use self::view_switcher::ViewSwitcher;
pub use self::view_switcher::ViewSwitcherBuilder;

mod view_switcher_bar;
pub use self::view_switcher_bar::ViewSwitcherBar;
pub use self::view_switcher_bar::ViewSwitcherBarBuilder;

mod view_switcher_title;
pub use self::view_switcher_title::ViewSwitcherTitle;
pub use self::view_switcher_title::ViewSwitcherTitleBuilder;

mod window;
pub use self::window::WindowBuilder;
pub use self::window::{Window, NONE_WINDOW};

mod window_handle;
pub use self::window_handle::WindowHandle;
pub use self::window_handle::WindowHandleBuilder;

mod enums;
pub use self::enums::CenteringPolicy;
pub use self::enums::DeckTransitionType;
#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
pub use self::enums::FlapFoldPolicy;
#[cfg(any(feature = "v1_2", feature = "dox"))]
#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_2")))]
pub use self::enums::FlapTransitionType;
pub use self::enums::HeaderGroupChildType;
pub use self::enums::LeafletTransitionType;
pub use self::enums::NavigationDirection;
pub use self::enums::SqueezerTransitionType;
pub use self::enums::ViewSwitcherPolicy;

pub mod functions;

#[doc(hidden)]
pub mod traits {
    pub use super::action_row::ActionRowExt;
    pub use super::combo_row::ComboRowExt;
    pub use super::deck::DeckExt;
    pub use super::expander_row::ExpanderRowExt;
    pub use super::header_bar::HeaderBarExt;
    pub use super::keypad::KeypadExt;
    pub use super::leaflet::LeafletExt;
    pub use super::preferences_group::PreferencesGroupExt;
    pub use super::preferences_page::PreferencesPageExt;
    pub use super::preferences_row::PreferencesRowExt;
    pub use super::preferences_window::PreferencesWindowExt;
    pub use super::search_bar::SearchBarExt;
    pub use super::swipeable::SwipeableExt;
}
